#!/usr/bin/python

########################################################################
# ns-revoke.py
# (c) by Michael Stroeder, michael.stroeder@inka.de
########################################################################

Version='0.5.3'

########################################################################
# CGI-BIN for reoking client certificates
# Input:
#
# PATH_INFO
# - Name of CA in openssl.cnf (section [ca] of openssl.cnf)
# QUERY_STRING
# - Serial number of certificate to revoke
#   max. 8 digits hexadecimal (32 Bit)
#
# Example:
#   ns-revoke.py/Persona?01
#   revokes client certificate with serial 0x01 of CA "Persona"
#
# The following checks are made to avoid denial of service attacks:
# - The client software must provide the client certificate.
# - The issuer of the client and the server certificates must match
########################################################################

import sys, os, string, re, htmlbase, openssl

from pycacnf import opensslcnf, pyca_section

# Wir lesen rein gar nix von Standardeingabe => gleich dicht machen
sys.stdin.close()

# Ein paar Umgebungsvariablen auslesen, welche der Apache liefert
request_method  = os.environ['REQUEST_METHOD']
query_string    = os.environ.get('QUERY_STRING','')

rm = (re.compile('[0-9a-fA-F]+(&yes)*')).match(query_string)

# Hier die ueblichen Paranoid-Pruefungen der Parameter
if request_method!='GET':
  # Skript nicht mit GET aufgerufen
  htmlbase.PrintErrorMsg('Wrong method.')
  sys.exit(0)

# Angabe der CA prfen
ca_name = os.environ.get('PATH_INFO','')[1:]
if not ca_name:
  htmlbase.PrintErrorMsg('No certificate authority.')
  sys.exit(0)
if not opensslcnf.data['ca'].has_key(ca_name):
  # CA-Definition nicht in openssl-Konfiguration enthalten
  htmlbase.PrintErrorMsg('Unknown certificate authority "%s"!' % ca_name)
  sys.exit(0)

ca_section=opensslcnf.data['ca'][ca_name]
ca_dir = opensslcnf.data[ca_section]['dir']

# Eintrag suchen lassen
entry = openssl.GetEntrybySerial(serialnumber)

# Header schreiben
print 'Content-type: text/html'

# Kein Zertifikat mit angegebener Nummer gefunden
if entry==[]:
  htmlbase.PrintErrorMsg('Certificate not found.')
  sys.exit(0)

if entry[openssl.DB_type]!=openssl.DB_TYPE_VAL:
  htmlbase.PrintErrorMsg('Certificate invalid.')
  sys.exit(0)

if not os.environ.has_key('SSL_CLIENT_DN') or \
   not os.environ.has_key('SSL_CLIENT_IDN'):
  htmlbase.PrintErrorMsg('No client certificate present.')
  sys.exit(0)

if os.environ['SSL_CLIENT_IDN']!=os.environ['SSL_SERVER_IDN']:
  htmlbase.PrintErrorMsg('Wrong issuer of client certificate.')
  sys.exit(0)

if os.environ['SSL_CLIENT_DN']!=entry[openssl.DB_name]:
  htmlbase.PrintErrorMsg('Wrong client certificate.')
  sys.exit(0)

if query_string[:len('SERIAL=')]!='SERIAL=':
  htmlbase.PrintHeader('Confirmation of certificate revocation.')
  print 'The following certificate will be revoked:<P>'
  print '<TABLE BORDER=1><TR><TD>%s</TD><TD>%s</TD></TR></TABLE><P>' % (
	 string.join(string.split(os.environ['SSL_CLIENT_DN'][1:],'/'),'<BR>'),
	 string.join(string.split(os.environ['SSL_CLIENT_IDN'][1:],'/'),'<BR>')
	)
  print """Are you really sure that you want to revoke your certificate?
The following reasons can make revoking necessary:
<UL>
  <LI>Your private key was compromised (stolen, the password was sniffed etc.)
  <LI>The content of the certificate attributes has become wrong.
</UL>
"""
  print '<FORM ACTION="%s?%s&yes"><INPUT TYPE=SUBMIT VALUE="Yes"><INPUT TYPE=RESET VALUE="No"></FORM>' % (os.environ['SCRIPT_NAME'],serialnumber)
  htmlbase.PrintFooter()

openssl.Revoke(serialnumber)
htmlbase.PrintHeader('Revoked certificate.')
print 'The following certificate was revoked by you:<P>'
print '<TABLE BORDER=1><TR><TD>%s</TD><TD>%s</TD></TR></TABLE><P>' % (
       string.join(string.split(os.environ['SSL_CLIENT_DN'][1:],'/'),'<BR>'),
       string.join(string.split(os.environ['SSL_CLIENT_IDN'][1:],'/'),'<BR>')
      )
  
sys.exit(0)

