#!/usr/bin/python

##############################################################################
# opensslcerts2ldif.py 0.1.2
#
# Alle gueltigen CA- und Client-Zertifikate einer Zertifizierungsstelle 
# in eine LDIF-Datei auf stdout schreiben.
# Alle relevanten Parameter werden aus einer openssl.cnf ausgelesen.
##############################################################################

##############################################################################
# Hier einige Variablen zur Konfiguration
##############################################################################

import openssl

# Umbenennungstabelle SSLeay-Attributname -> LDAP-Attributname
# Mu angepat werden, falls in SSLeay zustzliche OIDs verwendet werden
ldifattrname = {
                 'C':'c',
                 'ST':'st',
                 'L':'l',
                 'O':'o',
                 'OU':'ou',
                 'CN':'cn',
                 'I':'initials',
                 'uid':'uid',
                 'Email':'mail'
               }

dn_template = 'cn=%s,mail=%s,ou=%s,o=%s,c=%s'

# objectclass fr Personeneintrge
ldif_entryobjectclass = 'person'

# Pfadname der SSLeay-Konfigurationsdatei
openssl.cnf_filename = '/usr/local/ssl/lib/openssl.cnf'
# Pfadname des SSLeay-Programms
openssl_bin         = '/usr/local/ssl/bin/openssl'
# Pfadname des SSLeay-Programms
ldif_bin         = '/usr/local/bin/ldif'

##############################################################################
# Ab hier gibt es nix mehr zu konfigurieren
##############################################################################

import sys, os, string, charset, getopt, pipes

def BuildLDAPDN(openssldn):
  # DN aus Template zusammenbauen
  dntemplatelist = string.split(dn_template,',')
  dnlist = []
  for i in dntemplatelist:
    attr,format = string.split(i,'=',1)
    if openssldn.has_key(opensslattrname[attr]) and openssldn[opensslattrname[attr]]:
      dnlist.append(i % (openssldn[opensslattrname[attr]]))
  return string.join(dnlist,',')

def WriteCertificateAttribute(pemfilename,attributename,opensslparam='x509'):
  # openssl x509 eine DER-Datei auf stdout schreiben lassen
  pipe = os.popen('%s %s -inform pem -outform der -in %s | %s -b "%s"' % (openssl_bin,opensslparam,pemfilename,ldif_bin,attributename))
  s=pipe.readline()
  while s:
    sys.stdout.write(s)
    s=pipe.readline()

ldifattrnamekeys=ldifattrname.keys()
opensslattrname = {}
for i in ldifattrnamekeys:
  opensslattrname[ldifattrname[i]]=i
opensslattrnamekeys=opensslattrname.keys()

def findoption(options,paramname):
  for i in options:
    if i[0]==paramname:
      return i
  return ()

def PrintErrorMsg(ErrorMsg='',ErrorCode=1,NoExit=0):
  sys.stderr.write('*** Error: %s\n\n' % ErrorMsg)
  if NoExit:
    return
  else:
    sys.exit(ErrorCode)

def PrintUsage(ErrorMsg='',ErrorCode=1):
  script_name = string.split(sys.argv[0],os.sep)[-1]
  sys.stderr.write("""*** %s *** (C) by Michael Stroeder, 1998

usage: %s [-o objectclass] [-c SSLeay config] [-n name of CA]
          [-s openssl binary] [-l ldif binary]

Write all SSLeay-entries matching all filters to a LDIF file on stdout.
A filter has the form <attribute>=<value rexexp.> .

Options:

       -o objectclass
              Specify an objectclass which is used for the entries.
	      Default: %s

       -c SSLeay config
              Specify the pathname of the SSLeay config file.
	      Default: %s

       -n name of CA
              Specify the name of the CA in the SSLeay config file.
	      If this parameter is not present all CA definitions are used.

       -s openssl
              Specify the pathname of the SSLeay binary.
	      Default: %s

       -l ldif
              Specify the pathname of the ldif binary.
	      Default: %s

       -p
              Switch on creation of all certified persons entries with
	      attribute usercertificate;binary.
	      Does only make sense with a CA which certifies users.
	      Default: %s

""" % (script_name,script_name,ldif_entryobjectclass,\
       openssl.cnf_filename,openssl_bin,ldif_bin))
  if ErrorMsg:
    sys.stderr.write('*** Error: %s\n\n' % ErrorMsg)
  sys.exit(ErrorCode)

##############################################################################
# __main__()
##############################################################################

# Kommandozeile parsen
try:
  options,args=getopt.getopt(sys.argv[1:],'o:c:s:l:n:p',[])
except getopt.error:
  PrintUsage('Wrong options!')

# objectclass
try:
  dummy,ldif_entryobjectclass = findoption(options,'-o')
except: pass

# Pfadname der SSLeay-Konfigurationsdatei
try:
  dummy,openssl.cnf_filename = findoption(options,'-c')
except: pass
if not os.path.isfile(openssl.cnf_filename):
  PrintUsage('SSLeay configuration file "%s" not found.' % openssl.cnf_filename)

# Name der CA
try:
  dummy,ca_name = findoption(options,'-n')
except:
  ca_name=''

# Pfadname des SSLeay-Programms
try:
  dummy,openssl_bin = findoption(options,'-s')
except: pass
if not os.path.isfile(openssl_bin):
  PrintUsage('SSLeay binary "%s" not found.' % openssl_bin)

# Pfadname des LDIF-Programms
try:
  dummy,ldif_bin = findoption(options,'-l')
except: pass
if not os.path.isfile(ldif_bin):
  PrintUsage('LDIF binary "%s" not found.' % ldif_bin)

createusercertificate = findoption(options,'-p')

# Sonstige Argumente 
if args:
  PrintUsage('No additional arguments allowed.')

opensslcnf = openssl.GetAllSections()

# Liste der gewuenschten CA's bauen
ca_list=[]
if ca_name:
  for i in opensslcnf['ca'].keys():
    if ca_name==opensslcnf['ca'][i]:
      ca_list.append(ca_name)
  if not ca_list:
    PrintErrorMsg('CA "%s" not found in configuration file %s!\n' % (ca_name,openssl.cnf_filename),90)
else:
  for i in opensslcnf['ca'].keys():
    ca_list.append(opensslcnf['ca'][i])

olddatabases = []
oldcacerts = []

for ca_name in ca_list:

  # Die relevanten Parameter der openssl.cnf extrahieren
  # dir		= /usr/local/ssl/TestCA/Partner	# Where everything is kept
  # certs	= $dir/certs		# Where the issued certs are kept
  # crl_dir	= $dir/crl		# Where the issued crl are kept
  # database	= $dir/index.txt	# database index file.
  # new_certs_dir = $dir/newcerts	# default place for new certs.
  # certificate	= $dir/cacert.pem 	# The CA certificate
  # crl		= $dir/crl.pem 		# The current CRL

  # Basisverzeichnis der CA
  ca_dir = opensslcnf[ca_name]['dir']
  if not os.path.isdir(ca_dir):
    PrintErrorMsg('Base directory "%s" of CA "%s" not found.' % (ca_dir,ca_name))

  # Verzeichnis mit den Zertifikaten
  ca_certsdir = string.replace(opensslcnf[ca_name]['new_certs_dir'],'$dir',ca_dir)
  if not os.path.isdir(ca_certsdir):
    PrintErrorMsg('Certificate directory "%s" of CA "%s" not found.' % (ca_certsdir,ca_name))

  # DB-Index der Zertifikate
  openssl.db_filename = string.replace(opensslcnf[ca_name]['database'],'$dir',ca_dir)
  if not os.path.isfile(openssl.db_filename):
    PrintErrorMsg('Certificate database "%s" of CA "%s" not found.' % (openssl.db_filename,ca_name))

  # CA-Zertifikat
  ca_cacert = string.replace(opensslcnf[ca_name]['certificate'],'$dir',ca_dir)
  if not os.path.isfile(ca_cacert):
    PrintErrorMsg('Certificate file "%s" of CA "%s" not found.' % (ca_cacert,ca_name))

  nscerttype = opensslcnf[opensslcnf[ca_name]['x509_extensions']]['nsCertType']

  ###################################
  # Zertifizierungsstelle schreiben #
  ###################################

  if not ca_cacert in oldcacerts:

    cacertvalues=openssl.GetCertValues(ca_cacert)
    cadn = openssl.SplitDN(charset.iso2utf(charset.trans(cacertvalues['subject'],charset.t61,charset.iso)))

    dn=BuildLDAPDN(cadn)
    sys.stdout.write('dn:%s\n' % (dn))
    sys.stdout.write('objectclass: top\nobjectclass: certificationauthority\n')
    for i in opensslattrnamekeys:
      try:
	sys.stdout.write('%s: %s\n' % (i,cadn[opensslattrname[i]]))
      except KeyError: pass
    WriteCertificateAttribute(ca_cacert,'cacertificate;binary')

    # Aktuelle CRL schreiben
    ca_crl = string.replace(opensslcnf[ca_name]['crl'],'$dir',ca_dir)
    if not os.path.isfile(ca_crl):
      PrintErrorMsg('CRL file "%s" of CA "%s" not found.' % (ca_crl,ca_name),NoExit=1)
    else:
      WriteCertificateAttribute(ca_crl,'certificaterevocationlist;binary','crl')
    # Leerzeile zum Trennen der Eintraege
    sys.stdout.write('\n')

  ###################################
  # Zertifizierte Objekte schreiben #
  ###################################

  if createusercertificate and (not openssl.db_filename in olddatabases):

    olddatabases.append(openssl.db_filename)

    allcertusers = openssl.GetEntriesbyDN({'CN':'.*'},0,1)

    if not allcertusers:
      sys.stderr.write('No matching entries found in SSLeay database.\n')
      sys.exit(99)

    sys.stderr.write('Found %d matching and valid entries in SSLeay database.\n' % len(allcertusers))

    # Eintraege schreiben
    for certuser in allcertusers:

      openssldn = openssl.SplitDN(charset.iso2utf(charset.trans(certuser[openssl.DB_name],charset.t61,charset.iso)))

      pemfilename = os.path.join(ca_certsdir,certuser[openssl.DB_serial]+'.pem')
      if not os.path.isfile(pemfilename):
	sys.stderr.write('Certificate %s file not found!\n' % pemfilename)

      dn = BuildLDAPDN(openssldn)

      sys.stdout.write('dn:%s\n' % (dn))
      sys.stdout.write('objectclass: top\nobjectclass: strongauthenticationuser\n')
      sys.stdout.write('objectclass: %s\n' % ldif_entryobjectclass)
      for i in opensslattrnamekeys:
	try:
	  sys.stdout.write('%s: %s\n' % (i,openssldn[opensslattrname[i]]))
	except KeyError: pass
      WriteCertificateAttribute(pemfilename,'usercertificate;binary')

      # Leerzeile zum Trennen der Eintraege
      sys.stdout.write('\n')
